<?php
// $Id: taxonomy_treemenu.admin.inc,v 1.7 2009/04/08 18:07:39 rcrowther Exp $


/*******************
 * Hook functions
 ******************/
function taxonomy_treemenu_menu_update_submit($form, &$form_state) {
  $menu = $form_state['values'];
  //dpm($menu);
  $targets = array();
  foreach ($menu['targets'] as $target ) {
    if ($target != '') {
      $targets[] = $target;
      taxonomy_treemenu_rebuild($target);    
    }
  }
  //dpm($targets);
  $menus = implode(', ', $targets);
  $t_args = array('%menus' => $menus);
  drupal_set_message(t('The treemenu(s) %menus has been rebuilt', $t_args));
  $form_state['redirect'] = $menu['destination'];  
}

function taxonomy_treemenu_menu_update_validate($form, &$form_state) {
  $checked = FALSE;
  foreach ($form_state['values']['targets'] as $target) {
  //dpm($target);
    if (!empty($target)) {$checked = TRUE;}
  }
  //dpm('checked: ');
  //dvm( $checked);
  if (!$checked) {
    form_set_error('targets', t("Please choose at least one menu!"));    
  }
}

function taxonomy_treemenu_menu_update($form_state) {
  $form = array();
  $options = array();

  $tmd = variable_get('taxonomy_treemenu_data', array());
  foreach ($tmd as $menu_name => $value) {
    $options[$menu_name] = substr($menu_name ,strpos('-', $menu_name));
  }

  //we dont want the persistancy of a drupal warning message, but we do want it's immediacy, so we steal the css class.
  $form['message'] = array(
    '#value' => "<div class='messages warning'>" . t("Treemenus should update automatically. This form provides some disaster recovery. If subnitted, this form rebuilds the checked menu(s) completely and, in the worst case, will delete and rebuild all links. This may not be the problem - try emptying Drupal and browser caches, or a menu rebuild, first.") . "</div>",
  );
  
  $form['targets'] = array(
    '#type' => 'checkboxes',
    '#options' => $options,
    '#default value' => '',
    '#description' => t('Perform a full update on any treemenu individually.'),
  );
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Rebuild'),
  );
  $form['back'] = array(
  '#type' => 'value',
  '#value' => $_GET['destination'],
  );
  //dpm($_GET['destination']);
  //dpm($form_state);
  return $form;
}

/**
 * Function for generating a set of options for selecting a branch from the taxonomy.
 * Not the same options or keys as taxonomy_form_all()
 */
function taxonomy_treemenu_form_all($free_tags = 0) {
  $vocabularies = taxonomy_get_vocabularies();
  $options = array();
  $options['-:-'] = t('choose a branch here');
  foreach ($vocabularies as $vid => $vocabulary) {
    if ($vocabulary->tags && !$free_tags) { continue; }
    $vocab_path = $vid . ':';
    $options[$vocab_path . '0'] = $vocabulary->name . ' (vocab)';
    $tree = taxonomy_get_tree($vid);
    if ($tree && (count($tree) > 0)) {
    
      //$options[$vocabulary->name] = array();
      foreach ($tree as $term) {
        $options[$vocab_path . $term->tid] = str_repeat('-', $term->depth + 1) . $term->name;
        //$options[$vocabulary->name][$term->tid] = str_repeat('-', $term->depth) . $term->name;
      }
    }
  }
  return $options;
}


/***************************
 * Themes and preprocessors
 **************************/
function template_preprocess_tt_ui_edit_menu(&$variables) {
  //dpm('tt template preprocess edit:');
  drupal_add_js(drupal_get_path('module', 'taxonomy_treemenu') .'/js/form_data.js');
  $ttPrefs = array('ttPrefs' => $variables['element']['#pref_values']);
  drupal_add_js($ttPrefs, 'setting');
  drupal_add_css(drupal_get_path('module', 'taxonomy_treemenu') . '/css/taxonomy-treemenu-admin.css');
}


/**
 * Use the stock form-element theme.
 */
function theme_branch_selector_by_option($element) {
  return theme('form_element', $element, $element['#children']);
}

function theme_branch_selector_by_gui($element) {
  drupal_add_js(drupal_get_path('module', 'taxonomy_treemenu') .'/js/gui_selector.js');
  return theme('form_element', $element, $element['#children']);
}


function theme_taxonomy_admin_js_link($title, $id) {
//edit-tree-root-data
  return "<a href='#' onclick=\"updateTargetBranch('" . $id . "')\">" . check_plain(trim($title)) . "</a>";
}

function taxonomy_treemenu_taxonomy_tree_output() {
  $output = '';
  $items = taxonomy_treemenu_form_all();
  foreach($items as $id => $title) {
    $link = theme('taxonomy_admin_js_link', $title, $id);
    //dpm($link);
    $output .= theme('menu_item', $link, FALSE);
  }
  return $output ? theme('menu_tree', $output) : '';
}

function taxonomy_treemenu_taxonomy_full_admin_js_tree() {
  //static $taxonomy_treemenu_js_tree_output = array();
  //if (!isset($taxonomy_treemenu_js_tree_output)) {
    $taxonomy_treemenu_js_tree_output = taxonomy_treemenu_taxonomy_tree_output();
  //}
  return $taxonomy_treemenu_js_tree_output;
}


/***************************************************
 * Primary processes and themes for special elements
 ***************************************************/

/**
 * Generate a gui for selecting a branch, returning the path.
 */
 //now, the way we've done this, we do not have a full menu to call,
 //and Durpal doesn't supply one (which is why we are here in the first place)
 //Making a menu is kind of hard as a one off,
 //but a list should be possible by Drupal methods...
function expand_branch_selector_by_gui($element) {
  $element['tree_root_data'] = array(
    '#type' => 'hidden',
    '#default_value' => '-:-',
  );

  //Includes stolen CSS to make a 'required' asterix.
  // (we can't do that properly, as this selector uses a hidden field)
  //The result is real enough, as we do validate for value existance.
  $element['selector_display'] = array(
    '#prefix' => "<div><b>Select Branch:&nbsp</b>&nbsp<b>vocabulary:&nbsp</b><span id='branch-selector-display-vid'>no choice made</span>&nbsp<b>term:&nbsp</b><span id='branch-selector-display-tid'>no choice made</span><span class=\"form-required\">&nbsp*</span></div>",
    '#value' => taxonomy_treemenu_taxonomy_full_admin_js_tree(),
  );

  $element['#tree'] = TRUE;
  return $element;
}

/**
 * Generate a set of options for selecting a branch, returning the path.
 */
function expand_branch_selector_by_option($elements, $form_values) {
  $options = array();
  $paths = array();

  //get titles and info
  //$options['choose a branch here'] = taxonomy_treemenu_form_all();
  $options = taxonomy_treemenu_form_all();

  $elements['#type'] = 'select';
  $elements['#title'] = t('Select Branch');
  $elements['#default_value'] = '-:-';
  $elements['#options'] = $options;
  $elements['#tree'] = TRUE;
  //$elements['#description'] = t("On submiting, the chosen branch (and sub-branches, depending on the 'depth' setting) are turned into a custom menu.");
  return $elements;
}

//The theme is a template, which doesn't appear here.
function taxonomy_treemenu_tab($element) {
   //dpm('tt tab:');
  $element['#tree'] = TRUE;
  return $element;
}

//The theme is a template, which doesn't appear here.
// Uses (for display) #title (and #children).
// #prefs_options should be an unserialized array. It is JSONed and passed to
// the form.
function taxonomy_treemenu_options($element) {
  //dpm('tt options:');
  $element['#tree'] = TRUE;
  return $element;
}

// Doesn't take anything, just place inside a tab.
// Hiddens don't need themes (even if they use HTML in pre- or suffixes).
function taxonomy_treemenu_tab_display($element) {
  $id = end($element['#parents']);
  $element['#prefix'] = '<div id = "' . $id . '" class="tt-edit-tab-display">' . $element['#tab_value'].'</div>';
  $element['#type'] = 'hidden';
  $element['#default_value'] = "";
  return $element;
}


function taxonomy_treemenu_sort_order_selector($data) {
  if ($data == 'DESC') {
    $asc = "black";
    $desc ="blue";
  }
  else {
    $asc =" blue";
    $desc = "black";
  }
  $html_l = '<a class="tt-sort-order-asc ' . $asc . '">ASC</a> &nbsp';
  $html_r = '<a class="tt-sort-order-desc ' . $desc . '">DESC</a>';

  return $html_l . $html_r;
}

/*
 * A checkbox which includes a SPAN tag INSIDE the theme,
 * (not outside, as with a prefix). The SPAN can contain HTML elements.
 *
 * The SPAN tag would normally be styled with a float:right. The contents are
 * excluded from form builder action and, if those elements are intended to
 * be active, they are are there to be grabbed by javaScript on the form itself.
 *
 * Uses the usual suspects for checkboxes processing options, with these additions,
 *  - #options is an array of #return_value and other descriptive data. Less
 * flexible than the Drupal parameter(?).
 * #sub_type
 *   Specific declaration of extra HTML elements to be included
 *   in the extra checkboxes SPANs. Triggers various subfunctions.
 * #sub_type_id
 *   Should be unique, though is prefixed by 'chkbx-sub-'. Note that the usual
 *   Drupal parantage is not used here, as its getting lunatic. So assure that 
 *   your supplied IDs are unique. Use descendant secletors in any JS.
 * #sub_type_class
 *   Against the usual grain, we offer a class tag, as the contents may be very
 *   diffreent, and this makes them easier to style, as opposed to big descendant
 *   selectors. Prefixed by 'chkbx-sub-'.
 * #sub_type_default_value
 *   As it says, a default value for any HTML contents.
 */
//Doesn't have processing, just a modified theme. Wraps in the usual form element
//theme, as does Drupal's checkbox.
function theme_taxonomy_treemenu_checkbox($element) {
  //dpm('tt theme checkbox:');
  //dpm($element['#sub_type_id']);
  //dpm($element['#sub_type_class']);


  _form_set_class($element, array('form-checkbox'));


  $checkbox = '<input ';
  $checkbox .= 'type="checkbox" ';
  $checkbox .= 'name="'. $element['#name'] .'" ';
  $checkbox .= 'id="'. $element['#id'] .'" ' ;
  $checkbox .= 'value="'. $element['#return_value'] .'" ';
  $checkbox .= $element['#value'] ? ' checked="checked" ' : ' ';
  $checkbox .= drupal_attributes($element['#attributes']) .' />';


  if (!is_null($element['#title'])) {
    $checkbox = '<label class="option" for="'. $element['#id'] .'">'. $checkbox .' '. $element['#title'] .'</label>';
  }

  // Our stuff.
  switch ($element['#sub_type']) {
    case 'sort_order_selector':
        $div_contents = taxonomy_treemenu_sort_order_selector($element['#sub_type_default_value']);
        break;
    default:
       $div_contents = '';
  }

  $wrapper_html = '<span id="chkbx-sub-'. $element['#sub_type_id'] .'" class="chkbx-sub-'. $element['#sub_type_class'] .'">'. $div_contents .'</span>';

  $checkbox = $wrapper_html  . $checkbox;

  // Back to the theme.
  unset($element['#title']);
  return theme('form_element', $element, $checkbox);
}

// Just use the core checkboxes theme.
function theme_taxonomy_treemenu_checkboxes($element) {
  //dpm('tt theme js checkboxes:');
  return theme('checkboxes', $element);
}

/*
 * Checkboxes which include a SPAN tag INSIDE the theme,
 * (not outside, as with a prefix). The SPAN can contain HTML elements. See the
 * taxonomy_treemenu_checkbox for more details.
 *
 * Uses the usual suspects for checkboxes processing options, with these additions,
 *  - #options is an array of #return_value and other descriptive data. Less
 * flexible than the Drupal parameter.
 *  - #sub_type is a specific declaration of extra HTML elements and data to be
 * included in the extra checkboxes DIVs. It is also hardcoded with a data key to
 * retrieve data from #options specifically for the extra SPAN contents.
 */
function taxonomy_treemenu_expand_checkboxes($element) {
  //dpm('tt expand checkboxes:');
  //dpm($element);
  switch ($element['#sub_type']) {
    case 'sort_order_selector':
        $default_value_key = 'order';
        $att_tag = 'sort-order';
        break;
    default:
       $sub_type_data_key = '';
  }
  
  foreach ($element['#options'] as $key => $data) {
      $element[$key] = array(
        '#type' => 'taxonomy_treemenu_checkbox',
        '#sub_type' => $element['#sub_type'],
        '#processed' => TRUE, //?
        '#title' => $data['title'],
        '#description' => $data['description'],
        '#sub_type_default_value' => $data[$default_value_key],
        '#sub_type_id' => $att_tag .'-'. $key,
        '#sub_type_class' => $att_tag,
        '#return_value' => $key,
        '#default_value' => $data['active'],
      );
  }
  return $element;
}


function theme_depth($element) {
  return theme('form_element', $element, $element['#children']);
}

function expand_depth_selector($elements, $form_values) {
  $options = array('full depth');
  for ($i = 1; $i < 9; $i++) {
    $options[] = $i;
  }
  $elements['#type'] = 'select';
  $elements['#options'] = $options;  
  return $elements;
}


//strangely, they need explicit declaration of theme
//documentation says they don't,
//Drupal knows they are there, as they bail out without one
//but they are not called automatically...
function taxonomy_treemenu_elements() {
  $type = array();
  $type['branch_selector_by_gui'] = array('#input' => TRUE, '#process' => array('expand_branch_selector_by_gui') );
  $type['branch_selector_by_option'] = array('#input' => TRUE, '#process' => array('expand_branch_selector_by_option') );
  $type['taxonomy_treemenu_options'] = array('#input' => TRUE, '#process' => array('taxonomy_treemenu_options') );
  $type['taxonomy_treemenu_tab'] = array('#input' => TRUE, '#process' => array('taxonomy_treemenu_tab') );
  $type['taxonomy_treemenu_checkbox'] = array('#input' => TRUE);
  $type['taxonomy_treemenu_checkboxes'] = array('#input' => TRUE, '#process' => array('taxonomy_treemenu_expand_checkboxes') );
  $type['taxonomy_treemenu_tab_display'] = array('#input' => TRUE, '#process' => array('taxonomy_treemenu_tab_display') );
  $type['depth'] = array('#input' => TRUE, '#process' => array('expand_depth_selector'));
  return $type;
}


function taxonomy_treemenu_form_branch_selector() {    
  $branch_selector_type = variable_get('taxonomy_treemenu_selector_is_gui', FALSE);

  $selector['branch_selector'] = array(
    '#type' => 'fieldset',
    '#title' => t('Branch selector'),
    '#collapsible' => FALSE,
    '#description' => t("On submitting, the chosen branch (and sub-branches, depending on the 'depth' setting) are turned into a custom menu."),
    '#required' => TRUE,
  );

  $sector_submit_label = 'toggle branch selector to ';
  $sector_submit_label .= ($branch_selector_type) ? 'gui' : 'option list';  
  $selector['branch_selector']['change_selector'] = array(
    '#type' => 'submit',
    '#value' => $sector_submit_label,
    '#validate' => array('taxonomy_treemenu_edit_menu_change_selector'),
  );
  
  if (variable_get('taxonomy_treemenu_selector_is_gui', FALSE)) {
 	  $selector['branch_selector']['tree_root_data'] = array('#type' => 'branch_selector_by_option', '#required' => TRUE);
  }
  else {
    $selector['branch_selector']['tree_root_display'] = array('#type' => 'branch_selector_by_gui', '#required' => TRUE);       
  }
 
  return $selector;
}


/*******************************
 * Helper functions for options.
 *******************************/

/**
 * A full set of options, with everything set to off.
 * Use taxonomy_treemenu_default_options() to derive default information. Both
 * functions are intended for form use only. treemenu config data is stored as a
 * version stripped of non-active material.

 *
 * @param <type> $key
 * @return
 * A very full array of data to be used in fom cofiguration.
 */
function taxonomy_treemenu_base_options($key = 'all') {
  $base_options = array(
    filter => array(
    ),
    fields => array(
    ),
    sort => array(
      'title'   => array('title' => 'Title', 'description'   => 'Sort titles by alphabet.', 'active' => FALSE, 'order' => 'ASC'),
      'status'  => array('title' => 'Status', 'description'  => 'Wether published.', 'active' => FALSE, 'order' => 'ASC'),
      'sticky'  => array('title' => 'Sticky', 'description'  => 'Wether the node is sticky.', 'active' => FALSE, 'order' => 'ASC'),
      'created' => array('title' => 'Created', 'description' => 'When the node was created.', 'active' => FALSE, 'order' => 'ASC'),
      'changed' => array('title' => 'Changed', 'description' => 'When the node was last changed.', 'active' => FALSE, 'order' => 'ASC'),
      'nid'     => array('title' => 'Unordered', 'description' => 'Sort by nid (database reference). In time, this sort becomes caotic.', 'active' => FALSE, 'order' => 'ASC'),
      'type'    => array('title' => 'Type', 'description' => 'The type of a node. The database holds this as a reference number.', 'active' => FALSE, 'order' => 'ASC'),
    ),
  );
  return ($key == 'all') ? $base_options : $base_options[$key];
}


/**
 * A full set of options, with everything set to off.
 * Intended for form use only. Treemenu config data is stored as a version
 * stripped of non-active material.
 *
 * @return <type>
 */
function taxonomy_treemenu_default_options() {
   $options_base = taxonomy_treemenu_base_options();
   $options_base['sort']['nid']['active'] = TRUE;
   return $options_base;
}

/**
 * Change the user prefs to system options array, ready for a form.
 * Options are easily changed to prefs, they come out nearly right from the
 * forms. Not so this direction.
 */
function taxonomy_treemenu_prefs_to_options($db_options) {
  $user_options = unserialize($db_options);
  $options = taxonomy_treemenu_base_options();

  // Sort data loading
  foreach($user_options['sort'] as $key => $item) {
    $options['sort'][$key]['active'] = TRUE;
    $options['sort'][$key]['order'] = $item['order'];
  }
  return $options;
}

function taxonomy_treemenu_optionline_to_options($ops_str) {
  //"&nbsp status ASC,  created DESC,  nid ASC,  "
  //dpm('tt_optionline to options:');
  // Dispose of display spacing.
  $ops_str = preg_replace('/&nbsp\s*/', '', $ops_str);
  // Crush commas, replace spaces with commas.
  $ops_str = preg_replace('/(,\s+)|(\s+)/', ',', $ops_str);
  // Trim/chomp commas.
  $ops_str = preg_replace('/(^,)|(,$)/', '', $ops_str);
  //dvm($ops_str == '');
  $ops_array = ($ops_str == "") ? array('nid', 'ASC') : explode(',', $ops_str);

  return $ops_array;
}


/*
 * Make an associatve array out of an explosion, basicly.
 */
function taxonomy_treemenu_process_sort_optionline($ops_str) {
  //dpm('tt sort opsline:');
  $sort_ops = array();
  $ops_array = taxonomy_treemenu_optionline_to_options($ops_str);
  //dpm($ops_array);
  //dpm(count($ops_array));
  for($i = 0; $i < count($ops_array); ) {
    $sort_ops[$ops_array[$i]] = array('order' => $ops_array[$i + 1]);
    $i++;
    $i++;
  };
    //dpm($sort_ops);
  return $sort_ops;
}

// #?
function taxonomy_treemenu_options_set_checkboxes(&$options, $option_key, $new_choice_numeric_key) {
  $option_base_options = taxonomy_treemenu_base_options($option_key);
  //Reset the options to base.
  $options[$option_key] = $option_base_options;

  $i = 0;
  foreach($options[$option_key] as &$v) {
    if($i == $new_choice_numeric_key) {
      $v = TRUE;
      break;
    };
    $i++;
  }
}

/**
 *  Sets the options array at a given ordinal number, resetting all other elements.
 * 
 * @param <type> $options
 * @param <type> $option_key
 * @param <type> $new_choice_numeric_key
 */
function taxonomy_treemenu_options_set_radios(&$options, $option_key, $new_choice_numeric_key) {
  $option_base_options = taxonomy_treemenu_base_options($option_key);
  //Reset the options to base.
  $options[$option_key] = $option_base_options;

  $i = 0;
  foreach($options[$option_key] as &$v) {
    if($i == $new_choice_numeric_key) {
      $v = TRUE;
      break;
    };
    $i++;
  }
}


/**
 *  Returns the ordinal of the first option with value == TRUE.
 *
 * @param <type> $options
 * @param <type> $option_key
 * @param <type> $new_choice_numeric_key
 * @return <type>
 */
function taxonomy_treemenu_options_get_radios(&$options, $option_key) {
  $i = 0;
   foreach($options[$option_key] as $v) {
    if ($v == TRUE) {
      break;
    }
    $i++;
  }
  return $i;
}

// Unused for moment, should be in main body?
function taxonomy_treemenu_sort_prefs_to_options($prefs) {
  $sort_options = array();

  foreach($prefs as $pref) {

     switch ($pref) {
        case "alphabet (on title)":
            $order = 'n.title';
            break;
        case "date":
            $order = 'n.created';
            break;
        case "type":
            $order = 'n.type';
            break;
        default:
        //unordered?
           $order = 'n.nid';
    }
  }
  return $sort_options;
}



//should really use double submit, as i'm not sure this would run validate functions, etc.
//pulled in the edit form, rather than altering it,
// allows us to put the function in a separate file
// but does it run proper validate funtions and so forth?
// and not really compatible with 'edit' function in hook_alter().
// Yeah, but it works and its callable from a seperate file.
// Can't be called with an invalid menu_name, as we set these ourselves in the hook_menu.
//but we use out hook_load anyway. Will protect future code.
function taxonomy_treemenu_edit_menu($form_state, $type, $menu_name = '') {
  //dpm($menu_name);
  $form = array();
  $treemenu = array();
  $treemenu_options = taxonomy_treemenu_default_options();

  if ($type == 'add') {
    //set defaults, replace branch selector with a value.
    $treemenu = array(
      'vid'   => 0,
      'tid'   => 0,
      'depth' => 0,
      'nodes' => 0,
      'leaf_node_count' => 1,
      'link_roots_to_menutrees' => 1,
      'options' => $treemenu_options,
      'title' => 'New menu', 
    );

    $form = taxonomy_treemenu_form_branch_selector();
    //will be done automatically by the require, I think, but lets assert it.
    $form['#insert'] = TRUE;
  }
  else {
    //get current data, display useful values
    if(!$treemenu = taxonomy_treemenu_load($menu_name, TRUE)) {
      drupal_not_found();
      return;
    };

    // Load options, if there are any, else default.
    if (!empty($treemenu['options'])) {
        $treemenu_options = taxonomy_treemenu_prefs_to_options($treemenu['options']);
    }
    
    // Uncheck to use default options, whatever the current state of prefs.
    //$treemenu_options = taxonomy_treemenu_default_options();

     $form['display1'] = array(
       '#value' => "<p><b>Treemenu</b> grown from  " . 'Vid: ' . $treemenu['vid'] . ', Tid: '. $treemenu['tid'] . "</p>",
     );

     $form['display2'] = array(
       '#value'  =>  "<p><b>Machine name</b> used for addressing in urls: " . $menu_name . "</p>",
     );

  }


  
  if (module_exists('dhtml_menu')) {

      $form['dhtml_menu'] = array(
        '#default_value'  => !empty($treemenu['use_dhtml_menu_on_pages']),
        '#title'          => t('Use the DHTML effect on !menu_name pages', array('!menu_name' => $menu_name)),
        '#description'    => t('If you switch this on, then block DHTML is disabled - the DHTML gets confused if two similar menus share the same page. To use the DHTML effect in blocks, see the block configuration pages.'),
        '#type'           => 'checkbox',
       // '#weight'         => -3,
      );

  }

  
  $form['render_options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Treemenu render options'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('Treemenu has a special renderer. This is a little slower than core, but adds useful functions for customizing treemenus.'),
  );

      $form['render_options']['node_options'] = array(
        '#type' => 'fieldset',
        '#title' => t('Treemenu node options'),
        '#collapsible' => FALSE,
        '#description' => t('Options to display child nodes, or number of child nodes.'),
      );
      
        $form['render_options']['node_options']['nodes'] = array(
          '#type' => 'checkbox',
          '#title' => t('Show nodes'),
          '#default_value' => $treemenu['nodes'],
          '#description' => t('If showing, node links are always enabled.'),
        );
              
        $form['render_options']['node_options']['leaf_node_count'] = array(
        '#type' => 'checkbox',
        '#title' => t('Add node count to leaf terms'),
        '#default_value' => $treemenu['leaf_node_count'],
        //'#description' => t('If showing, node links are always enabled.'),
        );
    
      $form['render_options']['menu_root_options'] = array(
        '#type' => 'fieldset',
        '#title' => t('Treemenu root link options'),
        '#collapsible' => FALSE,
        '#description' => t('Modify the behaviour of treemenu roots.'),
      );
      
       $form['render_options']['menu_root_options']['link_roots_to_menutrees'] = array(
          '#title' => t('Link treemenus'),
          '#type' => 'checkbox',
          '#default_value' => $treemenu['link_roots_to_menutrees'],
          '#description' => t('If a link in this menu points to another treemenu root, then the link will page the treemenu directly (not page a list of terms or nodes). Only works if the menu is shown in a page.'),
        );


  $form['taxonomy_treemenu_options'] = array(
    '#type' => 'taxonomy_treemenu_options',
    '#title' => t('Options'),
    //We don't use these directly in this element or it's display,
    // but it's an appropriate place to store the data, e.g. for JSON.
    '#pref_values' => $treemenu_options,
    '#tree' => TRUE,
  );

    $form['taxonomy_treemenu_options']['tab1'] = array(
       '#type'           => 'taxonomy_treemenu_tab',
       '#title'          => t('sort nodes by:'),
    );

      $form['taxonomy_treemenu_options']['tab1']['tab1_display'] = array(
         '#type'           => 'taxonomy_treemenu_tab_display',
         '#tab_value'          => 'sort',
      );

      $form['taxonomy_treemenu_options']['tab1']['sort_options'] = array(
         '#type'           => 'taxonomy_treemenu_checkboxes',
         '#sub_type'       => 'sort_order_selector',
         '#options'        => $treemenu_options['sort'],
      );

/*
      $form['taxonomy_treemenu_options']['tab2'] = array(
         '#type'           => 'taxonomy_treemenu_tab',
         '#title'          => t('fields to add to node links:'),
      );
*/

  $form['depth'] = array(
    '#type' => 'depth',
    '#title' => t('Depth'),
    '#description' => t('...of menu. The tree root will never show, just descendant terms (and maybe nodes).'),
    '#default_value' => $treemenu['depth'], 
  );
 
  //pull in menu_edit_menu_submit()
  require_once(drupal_get_path('module', 'menu') .'/menu.admin.inc');
  $form = array_merge($form, menu_edit_menu($form_state, $type, $treemenu));

  /* Yes we do require these, but that requirement will validate
  before the branch selector, tripping errors, and annoying the user.
  So we'll validate these fields in the function, not automatically.
  */
  $form['title']['#required'] = FALSE;
  $form['menu_name']['#required'] = FALSE;
  
  /* ...but some warning. */
  $form['title']['#title'] .= "<span class=\"form-required\">*</span>";
  $form['menu_name']['#title'] .= "<span class=\"form-required\">*</span>";
  //dpm($treemenu);
  //dpm($form);

  return $form;  
}

function taxonomy_treemenu_edit_menu_change_selector($form, &$form_state) {
    $form_state['rebuild'] = TRUE;
    variable_set('taxonomy_treemenu_selector_is_gui', !(variable_get('taxonomy_treemenu_selector_is_gui', FALSE)));
}

/**
 * Validates the human and machine-readable names when adding or editing a menu.
 */
 //aw, stuff it. A near copy of menu.admin.incs validation.
 //some of this may be depreciated now?
function taxonomy_treemenu_edit_menu_validate($form, &$form_state) {
  $item = $form_state['values'];
  //check there are values at all
  //in title/menuname/hidden variables
  //dpm($item);
  if (strlen(trim($item['menu_name'])) == 0) {
    form_set_error('menu_name', t('The menu name is required.'));
  }
  if (strlen(trim($item['title'])) == 0) {
    form_set_error('title', t('The title name is required.'));
  }
  if ($item['tree_root_display']['tree_root_data'] == '-:-') {
    form_set_error('tree_root_display', t('A branch selection is required.'));
  }
  if ($item['tree_root_data'] == '-:-') {
    form_set_error('tree_root_data', t('A branch selection is required.'));
  } 
  if (preg_match('/[^a-z0-9-]/', $item['menu_name'])) {
    form_set_error('menu_name', t('The menu name may only consist of lowercase letters, numbers, and hyphens.'));
  }
  if (strlen($item['menu_name']) > MENU_MAX_MENU_NAME_LENGTH_UI) {
    form_set_error('menu_name', format_plural(MENU_MAX_MENU_NAME_LENGTH_UI, "The menu name can't be longer than 1 character.", "The menu name can't be longer than @count characters."));
  }
  if ($form['#insert']) {
    // We will add 'menu-' to the menu name to help avoid name-space conflicts.
    $item['menu_name'] = 'menu-'. $item['menu_name'];
    if (db_result(db_query("SELECT menu_name FROM {menu_custom} WHERE menu_name = '%s'", $item['menu_name'])) ||
      db_result(db_query_range("SELECT menu_name FROM {menu_links} WHERE menu_name = '%s'", $item['menu_name'], 0, 1))) {
      form_set_error('menu_name', t('The menu already exists.'));
    }
  }
}


/**
 * Special submit handler calls menu_edit_menu_submit() first,
 * thus creating initial framework,
 * then calls this, which adds submenu data and submenu items.
 */
 // I tried. I tried using hook_form_alter, but its too lightweight. Either it fails to address stuff in other files,
 //loosing callbacks and form elements, or you stick your entire admin in the main file, and have to jiggle like crazy.
 //I tried redirection (drupal_goto) from hook_alter, but once the form handlers have the form, they're off, 
 //and you can't cancel the request, not fully - and you've built a form only to throw it away.
 //I considered a seperate structure, but that would be awful for the user - 'use this form, not this one'. 
  //The current idea is to hijack the menu-edit button by using specific paths, which are built in hook_menu. 
  //This is not elegant, but it effective, and Drupalish.
  //The add 'Add' version gets it's own link, to appear in the menu pages.
  //The end result callbacks here, where we require the menu edit forms.
  //Anyone got any tidier ideas? (We could use file attributes to pull in, and drupal submits, I suppose). 
function taxonomy_treemenu_edit_menu_submit($form, &$form_state) {
  //dpm('tt edit submit:');

  //return;
  //pull in menu_edit_menu_submit()
  require_once(drupal_get_path('module', 'menu') .'/menu.admin.inc');
  menu_edit_menu_submit($form, &$form_state);
  
  //restore the relevant variables
  $menu = $form_state['values'];
  if ($form['#insert']) {
    $menu['menu_name'] = 'menu-'. $menu['menu_name'];
  }


  // Retrieve our data, if any.
  //$tmd = variable_get('taxonomy_treemenu_data', array());
  $tmd = taxonomy_treemenu_all_load();

  // Failsafe for settings data load.
  if (empty($tmd)) {
    $tmd = array($menu['menu_name'] => array());
  }

  // Test area. Unset wonky variables here!
  //unset($tmd['menu-redtree-tt']);
  //$root_ids = array(3, 0);
  //dpm($menu);
  //$tmd = array($menu['menu_name'] => array());
  //dpm($tmd[$menu['menu_name']]);


  // Some preparatory work, then an array of data.

  //If 'use_dhtml_menu_on_pages' is on then we switch 'use_dhtml_menu_in_blocks'
  // off, or the DHTML gets confused.
  $use_dhtml_in_blocks = ($menu['dhtml_menu']) ? 0 : $tmd[$menu['menu_name']]['use_dhtml_menu_in_blocks'];

  //Configure options by retrieving material.
  $options = taxonomy_treemenu_base_options();
  //taxonomy_treemenu_options_set_radios($options, 'sort', $menu['taxonomy_treemenu_options']['sort']);
 
  $options['sort'] = taxonomy_treemenu_process_sort_optionline($menu[taxonomy_treemenu_options][tab1][tab1_display]);
  //dpm($options);

  $new_menu_data = array(
    //'vid'   => $root_ids[0],
    //'tid'   => $root_ids[1],
    'depth' => $menu['depth'],
    'nodes' => $menu['nodes'],
    'leaf_node_count' => $menu['leaf_node_count'],
    'use_dhtml_menu_in_blocks' => $use_dhtml_in_blocks ,
    'use_dhtml_menu_on_pages' => $menu['dhtml_menu'],
    'link_roots_to_menutrees' => $menu['link_roots_to_menutrees'],
    'options'=> serialize($options),
    'options_changed' => 1, //?
    );


  if ($form['#insert']) {
    $tmd[$menu['menu_name']] = $new_menu_data;

    // Retrieve tree root data
    $tree_root_data = variable_get('taxonomy_treemenu_selector_is_gui', FALSE) ? $menu['tree_root_data'] : $menu['tree_root_display']['tree_root_data'];
    $root_ids = explode(':', $tree_root_data);

    $tmd[$menu['menu_name']]['vid'] = $root_ids[0];
    $tmd[$menu['menu_name']]['tid'] = $root_ids[1];

    //...and build the subtree links
    //taxonomy_treemenu_create_sublinks($root_ids[0], $root_ids[1], $menu['menu_name'], $menu['depth'], $menu['nodes']);
    taxonomy_treemenu_create_sublinks($root_ids[0], $root_ids[1], $menu['menu_name'], $menu['depth']);
  }
  else {
    $tmd[$menu['menu_name']] = array_merge($tmd[$menu['menu_name']], $new_menu_data);

    //Some backwards compatibility cleanup (V4).
    unset($tmd[$menu['menu_name']]['disable_treemenu_links']);
    unset($tmd[$menu['menu_name']]['enable_root_links']);
    unset($tmd[$menu['menu_name']]['enable_tree_leaves']);
    unset($tmd[$menu['menu_name']]['_disable_branches']);
    unset($tmd[$menu['menu_name']]['_disable_leaves']);
  }

  variable_set('taxonomy_treemenu_data', $tmd);
  //dpm($root_ids[0] .' : '. $root_ids[1].' : '. $menu['menu_name'].' : '. $menu['depth'].' : '. $menu['nodes']);
  
  //TODO: what do you mean by this? if you change the root, do you mean?
  //This we now know, can cause problems.
  //(but?)needed for the admin, or the link for the new menu will notappear.
  menu_rebuild();
  //menu_cache_clear($menu_name);
}
